<?php

/**
 * @file
 * Contains \Drupal\yamlform\Controller\YamlFormPluginElementController.
 */

namespace Drupal\yamlform\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\yamlform\YamlFormElementManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for all YAML form elements.
 */
class YamlFormPluginElementController extends ControllerBase {

  /**
   * A element manager.
   *
   * @var \Drupal\Core\Render\ElementInfoManagerInterface
   */
  protected $elementManager;

  /**
   * A YAML form element plugin manager.
   *
   * @var \Drupal\yamlform\YamlFormElementManagerInterface
   */
  protected $yamlFormElementManager;

  /**
   * Constructs a YamlFormPluginBaseController object.
   *
   * @param \Drupal\Core\Render\ElementInfoManagerInterface $element_manager
   *   A element plugin manager.
   * @param \Drupal\yamlform\YamlFormElementManagerInterface $yamlform_element_manager
   *   A YAML form element plugin manager.
   */
  public function __construct(ElementInfoManagerInterface $element_manager, YamlFormElementManagerInterface $yamlform_element_manager) {
    $this->elementManager = $element_manager;
    $this->yamlFormElementManager = $yamlform_element_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.element_info'),
      $container->get('plugin.manager.yamlform.element')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function index() {

    $yamlform_form_element_rows = [];
    $element_rows = [];

    $element_plugin_definitions = $this->elementManager->getDefinitions();
    foreach ($element_plugin_definitions as $element_plugin_id => $element_plugin_definition) {
      if ($this->yamlFormElementManager->hasDefinition($element_plugin_id)) {
        /** @var \Drupal\yamlform\YamlFormElementInterface $yamlform_element */
        $yamlform_element = $this->yamlFormElementManager->createInstance($element_plugin_id);
        $yamlform_element_plugin_definition = $this->yamlFormElementManager->getDefinition($element_plugin_id);

        $default_format = $yamlform_element->getDefaultFormat();
        $format_names = array_keys($yamlform_element->getFormats());
        $formats = array_combine($format_names, $format_names);
        if (isset($formats[$default_format])) {
          $formats[$default_format] = '<b>' . $formats[$default_format] . '</b>';
        }

        $yamlform_form_element_rows[$element_plugin_id] = [
          $element_plugin_id,
          $yamlform_element_plugin_definition['label'],
          $yamlform_element_plugin_definition['multiline'] ? t('Yes') : t('No'),
          $yamlform_element_plugin_definition['multiple'] ? t('Yes') : t('No'),
          ['data' => ['#markup' => implode('; ', $formats)]],
          $element_plugin_definition['provider'],
          $yamlform_element_plugin_definition['provider'],
        ];
      }
      else {
        $element_rows[$element_plugin_id] = [
          $element_plugin_id,
          $element_plugin_definition['provider'],
        ];
      }
    }

    $build = [];

    ksort($yamlform_form_element_rows);
    $build['yamlform_elements'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Name'),
        $this->t('Label'),
        $this->t('Multiline'),
        $this->t('Multiple'),
        $this->t('Formats'),
        $this->t('Provided by'),
        $this->t('Integrated by'),
      ],
      '#rows' => $yamlform_form_element_rows,
    ];

    ksort($element_rows);
    $build['elements'] = [
      '#type' => 'details',
      '#title' => $this->t('Additional elements'),
      '#description' => t('Below are element that are available to a YAML form but do not have YAML form element plugin and/or require any additional integration'),
      'table' => [
        '#type' => 'table',
        '#header' => [
          $this->t('Name'),
          $this->t('Provided by'),
        ],
        '#rows' => $element_rows,
      ],
    ];

    return $build;
  }

}
