<?php

/**
 * @file
 * Contains \Drupal\yamlform\Plugin\YamlFormElement\Captcha.
 */

namespace Drupal\yamlform\Plugin\YamlFormElement;
use Drupal\yamlform\YamlFormSubmissionInterface;

/**
 * Provides a 'captcha' element.
 *
 * @YamlFormElement(
 *   id = "captcha",
 *   label = @Translation("captcha")
 * )
 */
class Captcha extends ContainerBase {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    $element['#access'] = !\Drupal::currentUser()->hasPermission('skip CAPTCHA');
    parent::prepare($element, $yamlform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    // Remove all captcha related keys from the YAML form submission's data.
    $key = $element['#key'];
    $data = $yamlform_submission->getData();
    unset($data[$key]);
    // @see \Drupal\captcha\Element\Captcha
    $sub_keys = ['sid', 'token', 'response'];
    foreach ($sub_keys as $sub_key) {
      unset($data[$key . '_' . $sub_key]);
    }
    $yamlform_submission->setData($data);
  }

}
