<?php

/**
 * @file
 * Contains \Drupal\yamlform\Plugin\YamlFormElement\YamlFormMarkup.
 */

namespace Drupal\yamlform\Plugin\YamlFormElement;

use Drupal\Core\Mail\MailFormatHelper;
use Drupal\yamlform\YamlFormSubmissionInterface;

/**
 * Provides a 'yamlform_markup' element.
 *
 * @YamlFormElement(
 *   id = "yamlform_markup",
 *   label = @Translation("YAML form markup")
 * )
 */
class YamlFormMarkup extends ContainerBase {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    // Hide markup element is it should be only displayed on 'view'.
    if (isset($element['#display_on']) && $element['#display_on'] == 'view') {
      $element['#access'] = FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildHtml(array &$element, $value, array $options = []) {
    // Hide markup element is it should be only displayed on 'form'.
    if (isset($element['#display_on']) && $element['#display_on'] == 'form') {
      $element['#access'] = FALSE;
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function buildText(array &$element, $value, array $options = []) {
    // Hide markup element is it should be only displayed on 'form'.
    if (isset($element['#display_on']) && $element['#display_on'] == 'form') {
      $element['#access'] = FALSE;
    }

    if (isset($element['#markup'])) {
      $element['#markup'] = MailFormatHelper::htmlToText($element['#markup']);
    }

    return $element;
  }

}
